package com.yidianling.ydlcommon.data

import com.ydl.ydlnet.YDLHttpUtils
import com.yidianling.common.tools.RxAppTool
import com.yidianling.common.tools.RxDeviceTool
import com.yidianling.ydlcommon.base.BaseApplication
import com.yidianling.ydlcommon.data.http.*
import com.yidianling.ydlcommon.data.http.params.*
import com.yidianling.ydlcommon.data.http.response.Balance
import com.yidianling.ydlcommon.data.http.response.Recharge
import com.yidianling.ydlcommon.data.http.response.TestAddOrderBean
import com.yidianling.ydlcommon.data.http.response.WXPay
import com.yidianling.ydlcommon.actions.pay.redpacket.RedPacketBean
import com.yidianling.ydlcommon.actions.pay.redpacket.RedPacketParam
import com.yidianling.ydlcommon.router.YdlCommonOut
import com.yidianling.ydlcommon.router.YdlCommonRouterManager
import com.yidianling.ydlcommon.ui.confideredpacket.RedListParam
import io.reactivex.Observable
import io.reactivex.schedulers.Schedulers
import okhttp3.MediaType
import okhttp3.RequestBody

/**
 * author : Zhangwenchao
 * e-mail : zhangwch@yidianling.com
 * time   : 2018/02/05
 */
class PlatformHttpImpl private constructor() : IPlatformHttp {

    companion object {
        fun getInstance(): PlatformHttpImpl {
            return Holder.INSTANCE
        }

        fun clearYdlCommonApi() {
            Holder.INSTANCE.api = null
        }
    }

    private var api: YdlCommonApi? = null

    private fun getYdlCommonApi(): YdlCommonApi {
        if (api == null) {
            api = YDLHttpUtils.obtainApi(YdlCommonApi::class.java)
        }
        return api!!
    }


    override fun uploadLog(param: LogParam): Observable<BaseResponse<Any>> {
//        return RxUtils.mapObservable(param)
//                .flatMap { api.uploadLog(it) }
        return Observable.just(param)
                .compose(RxUtils.netCheck())
                .subscribeOn(Schedulers.io())
                .map {
                    val map = HashMap<String, RequestBody>()
                    val mediaType = MediaType.parse("text/plain")
                    map.put("apiurl", RequestBody.create(mediaType, it.apiurl))
                    map.put("os_type", RequestBody.create(mediaType, it.os_type.toString()))
                    map.put("errorCode", RequestBody.create(mediaType, it.errorCode.toString()))
                    map.put("errorMsg", RequestBody.create(mediaType, it.errorMsg))
                    map.put("ffrom", RequestBody.create(mediaType, YdlCommonOut.getChannelName()))
                    map.put("isFromApp", RequestBody.create(mediaType, "1"))
                    map.put("osBuild", RequestBody.create(mediaType, """${RxDeviceTool.getBuildBrandModel()},${RxDeviceTool.getSDKVersionName()},${RxAppTool.getAppVersionName(
                        BaseApplication.getApp())}"""))
                    map.put("ts", RequestBody.create(mediaType, (System.currentTimeMillis() / 1000).toString()))
                    map.put("version", RequestBody.create(mediaType, RxAppTool.getAppVersionName(BaseApplication.getApp())))
                    val userInfo = YdlCommonRouterManager.getYdlCommonRoute()?.getUserInfo()
                    if (userInfo != null) {
                        map.put("uid", RequestBody.create(mediaType, userInfo.userId))
                        map.put("accessToken", RequestBody.create(mediaType, userInfo.token))
                    }
                    val fileBody = RequestBody.create(MediaType.parse("multipart/form-data"), it.file)
                    map.put("""uploadlog"; filename="${it.file.name}""", fileBody)
                    map
                }
                .flatMap { getYdlCommonApi().uploadLog(it) }
    }

    override fun wxRecharge(param: WXRechargeIdParam): Observable<BaseResponse<WXPay>> {
        return RxUtils.mapObservable(param)
                .flatMap { getYdlCommonApi().wxRecharge(it) }
    }

    override fun getBalance(param: BalanceParam): Observable<BaseResponse<Balance>> {
        return RxUtils.mapObservable(param)
                .flatMap { getYdlCommonApi().getBalance(it) }
    }

    override fun recharge(param: RechargeParam): Observable<BaseResponse<Recharge>> {
        return RxUtils.mapObservable(param)
                .flatMap { getYdlCommonApi().recharge(it) }
    }

    override fun pay(param: BalancePayParam): Observable<BaseResponse<Any>> {
        return RxUtils.mapObservable(param)
                .flatMap { getYdlCommonApi().pay(it) }
    }

    override fun wxPay(param: PayParam.WxPayParam): Observable<BaseResponse<WXPay>> {
        return RxUtils.mapObservable(param)
                .flatMap { getYdlCommonApi().wxPay(it) }
    }

    override fun aliPay(param: PayParam.AliPayParam): Observable<BaseResponse<Recharge>> {
        return RxUtils.mapObservable(param)
                .flatMap { getYdlCommonApi().aliPay(it) }
    }

    override fun getCoupon(): Observable<BaseResponse<ArrayList<RedPacketBean>>> {
        return RxUtils.mapObservable(
            RedPacketParam(
                ""
            )
        )
                .flatMap { getYdlCommonApi().getCoupon(it) }
    }

    override fun addOrder(param: AddOrderParam): Observable<BaseResponse<TestAddOrderBean>> {
        return RxUtils.mapObservable((param))
                .flatMap { getYdlCommonApi().testAddOrder(it) }
    }

    override fun getRedList(param: RedListParam): Observable<BaseResponse<Any>> {
        return RxUtils.mapObservable((param))
                .flatMap { getYdlCommonApi().getRedList(it) }
    }

    private object Holder {
        internal val INSTANCE = PlatformHttpImpl()
    }
}